/* 
   Author: Joel W. Reed (joelreed@yahoo.com).  Copyright 1999.
   This file is distributed  under the Artistic License.
   See the file LICENSE for details.
*/

#ifndef _nn_structs_h_
#define _nn_structs_h_

#include <string>
#include <vector>

typedef double fpnum_t; 
const int MAX_TAG_SIZE = 512;

struct unit
{
	unit(const char* t="<internal_node>", fpnum_t v=0.0);
	
	fpnum_t _value, _delta;
	std::string _tag;
};

struct weight
{
	weight(fpnum_t v=1.0, unit* f=0, unit* s=0);
	fpnum_t _value;
	unit *_first, *_second;
};

struct unit_layer : public std::vector<unit>
{
	unit_layer(int sz=0) : std::vector<unit>(sz) {}
};

struct weight_layer : public std::vector<weight>
{
};

struct pattern
{
	bool load(const char* fn=0);
	unit_layer _input;
	unit_layer _output;
};

struct network 
{
	typedef std::vector<unit_layer> unit_layers_t;
	typedef std::vector<weight_layer> weight_layers_t;

	network();

	bool initialize(const std::vector<std::string>&, const std::vector<std::string>&, const std::vector<int>& dims);
	bool load(const char* fn=0);
	bool save(const char* fn=0) const;
	bool learn(const unit_layer&);
	bool process(const unit_layer &);

	protected: std::vector<unit_layer> _unit_layers;
	protected: std::vector<weight_layer> _weight_layers;
};

#endif
